
# commandArgs()[1] = HTSeq count table
# commandArgs()[2] = prefix of output files
# commandArgs()[3] = num. of replicates
# commandArgs()[4] = threshold

#----- library -----#
library(DESeq)
#-------------------#


#----- input -----#
args <- commandArgs(trailingOnly = TRUE)
inputFile <- args[1]
outputPrefix <- args[2]
NUM <- as.numeric(args[3])
threshold <- as.numeric(args[4])
data <- read.table(inputFile, header = T, row.names = 1)
outputNormalizedCount <- paste(outputPrefix, ".normalized_count", sep = "")
outputNormalizedMean <- paste(outputPrefix, ".normalized_mean", sep = "")
outputTestResults <- paste(outputPrefix, ".results", sep = "")
outputSignResults <- paste(outputPrefix, ".significant", sep = "")
#-----------------#


#----- calc size factors -----#
cds <- newCountDataSet(data, rep(c("A", "B", "C", "D"), each = NUM))
cds <- estimateSizeFactors(cds)
normalizedCount <- counts(cds, normalized = TRUE)
#-----------------------------#


#----- calc mean expression levels for each condition -----#
normalizedMean <- data.frame(A = rep(0, times = nrow(normalizedCount)), B = rep(0, times = nrow(normalizedCount)), C = rep(0, times = nrow(normalizedCount)), D = rep(0, times = nrow(normalizedCount)), row.names = rownames(normalizedCount))

for(i in 1 : nrow(normalizedCount)){
      for(j in 0 : 3){
      	    SUM = 0
	    for(k in 1 : NUM){
	    	  SUM = SUM + normalizedCount[i, (j * NUM + k)]
	    }
	    normalizedMean[i, (j + 1)] <- SUM / NUM
      }
}
#----------------------------------------------------------#


#----- estimate dispersions and perform statistical tests -----#
cds <- estimateDispersions(cds, method="blind", sharingMode="fit-only", fitType="local")
AB <- nbinomTest(cds, "A", "B")
AC <- nbinomTest(cds, "A", "C")
AD <- nbinomTest(cds, "A", "D")
BC <- nbinomTest(cds, "B", "C")
BD <- nbinomTest(cds, "B", "D")
CD <- nbinomTest(cds, "C", "D")
#--------------------------------------------------------------#


#----- calc q-values -----#
ab <- data.frame(transform(AB, sample1 = "A", sample2 = "B"), row.names = paste(rownames(AB), "_AB", sep = ""))
ac <- data.frame(transform(AC, sample1 = "A", sample2 = "C"), row.names = paste(rownames(AC), "_AC", sep = ""))
ad <- data.frame(transform(AD, sample1 = "A", sample2 = "D"), row.names = paste(rownames(AD), "_AD", sep = ""))
bc <- data.frame(transform(BC, sample1 = "B", sample2 = "C"), row.names = paste(rownames(BC), "_BC", sep = ""))
bd <- data.frame(transform(BD, sample1 = "B", sample2 = "D"), row.names = paste(rownames(BD), "_BD", sep = ""))
cd <- data.frame(transform(CD, sample1 = "C", sample2 = "D"), row.names = paste(rownames(CD), "_CD", sep = ""))
all <- rbind(ab, ac, ad, bc, bd, cd)
q <- p.adjust(all$pval, method = "BH")
result <- transform(all, qval = q)
#-------------------------#


#----- output results -----#
out <- file(outputNormalizedCount, "w")
write.table(normalizedCount, out, col.names = F, row.names = T, append = F, quote = F, sep = "\t")
close(out)

out <- file(outputNormalizedMean, "w")
write.table(normalizedMean, out, col.names = F, row.names = T, append = F, quote = F, sep = "\t")
close(out)

out <- file(outputTestResults, "w")
write.table(result, out, col.names = T, row.names = F, quote = F, sep = "\t")
close(out)

out <- file(outputSignResults, "w")
write.table(subset(result, qval < threshold), out, col.names = T, row.names = F, quote = F, sep = "\t")
close(out)
#--------------------------#
