
# commandArgs()[1] = HTSeq count table
# commandArgs()[2] = prefix of output files
# commandArgs()[3] = num. of replicates
# commandArgs()[4] = threshold

#----- library -----#
library(edgeR)
#-------------------#


#----- input -----#
args <- commandArgs(trailingOnly = TRUE)
inputFile <- args[1]
outputPrefix <- args[2]
NUM <- as.numeric(args[3])
threshold <- as.numeric(args[4])
data <- read.table(inputFile, header = T, row.names = 1)
outputNormalizedCount <- paste(outputPrefix, ".normalized_count", sep = "")
outputNormalizedMean <- paste(outputPrefix, ".normalized_mean", sep = "")
outputTestResults <- paste(outputPrefix, ".results", sep = "")
outputSignResults <- paste(outputPrefix, ".significant", sep = "")
#-----------------#


#----- normalization and dispersion  -----#
group <- factor(rep(c("A", "B", "C", "D"), each = NUM))
d <- DGEList(counts = data, group = group)
d <- calcNormFactors(d)
if(NUM == 1){
       	d <- estimateGLMCommonDisp(d, method = "deviance", robust = T, subset = NULL)
}else{
	d <- estimateCommonDisp(d)
	d <- estimateTagwiseDisp(d)
}
#-----------------------------------------#


#----- statistical test -----#
AB <- exactTest(d, pair = c("A", "B"))
AC <- exactTest(d, pair = c("A", "C"))
AD <- exactTest(d, pair = c("A", "D"))
BC <- exactTest(d, pair = c("B", "C"))
BD <- exactTest(d, pair = c("B", "D"))
CD <- exactTest(d, pair = c("C", "D"))
#----------------------------#


#----- convert to data frame -----#
AB_d <- as.data.frame(topTags(AB, n = nrow(data)))
AC_d <- as.data.frame(topTags(AC, n = nrow(data)))
AD_d <- as.data.frame(topTags(AD, n = nrow(data)))
BC_d <- as.data.frame(topTags(BC, n = nrow(data)))
BD_d <- as.data.frame(topTags(BD, n = nrow(data)))
CD_d <- as.data.frame(topTags(CD, n = nrow(data)))
#---------------------------------#


#----- calc q-values -----#
ab <- data.frame(transform(AB_d, sample1 = "A", sample2 = "B"), row.names = paste(rownames(AB_d), "_AB", sep = ""))
ac <- data.frame(transform(AC_d, sample1 = "A", sample2 = "C"), row.names = paste(rownames(AC_d), "_AC", sep = ""))
ad <- data.frame(transform(AD_d, sample1 = "A", sample2 = "D"), row.names = paste(rownames(AD_d), "_AD", sep = ""))
bc <- data.frame(transform(BC_d, sample1 = "B", sample2 = "C"), row.names = paste(rownames(BC_d), "_BC", sep = ""))
bd <- data.frame(transform(BD_d, sample1 = "B", sample2 = "D"), row.names = paste(rownames(BD_d), "_BD", sep = ""))
cd <- data.frame(transform(CD_d, sample1 = "C", sample2 = "D"), row.names = paste(rownames(CD_d), "_CD", sep = ""))
all <- rbind(ab, ac, ad, bc, bd, cd)
q <- p.adjust(all$PValue, method = "BH")
result <- transform(all, qval = q)
#-------------------------#


#----- calc mean expression levels for each condition -----#
normalizedCount <- cpm(d, normalized.lib.size = T)
normalizedMean <- data.frame(A = rep(0, times = nrow(normalizedCount)), B = rep(0, times = nrow(normalizedCount)), C = rep(0, times = nrow(normalizedCount)), D = rep(0, times = nrow(normalizedCount)), row.names = rownames(normalizedCount))

for(i in 1 : nrow(normalizedCount)){
      for(j in 0 : 3){
      	    SUM = 0
	    for(k in 1 : NUM){
	    	  SUM = SUM + normalizedCount[i, (j * NUM + k)]
	    }
	    normalizedMean[i, (j + 1)] <- SUM / NUM
      }
}
#----------------------------------------------------------#


#----- output results -----#
out <- file(outputNormalizedCount, "w")
write.table(normalizedCount, out, col.names = F, row.names = T, append = F, quote = F, sep = "\t")
close(out)

out <- file(outputNormalizedMean, "w")
write.table(normalizedMean, out, col.names = F, row.names = T, append = F, quote = F, sep = "\t")
close(out)

out <- file(outputTestResults, "w")
write.table(result, out, col.names = T, row.names = T, quote = F, sep = "\t")
close(out)

out <- file(outputSignResults, "w")
write.table(subset(result, qval < threshold), out, col.names = T, row.names = T, quote = F, sep = "\t")
close(out)
#--------------------------#
